package datahub.tools;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.tika.Tika;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

import com.opencsv.CSVReader;

import datahub.model.*;


/*
 * Class for obtaining data from raw dataset files
 */
public class DataGatherer {

	private static String UPLOADED_FOLDER = "files/users/";
	
	public static ArrayList<DatasetFile> getDatasetFilesData(String folder){
		ArrayList<DatasetFile> files = new ArrayList<DatasetFile>();
		
		File folderF = new File(folder);
		
		if(folderF != null && folderF.isDirectory()) {

			for (File fileEntry : folderF.listFiles()) {
		        if (fileEntry.isDirectory()) {
		            files.addAll(getDatasetFilesData(fileEntry.getAbsolutePath()));
		        } else {
		            files.add(getFileData(fileEntry));
		        }
		    }
		}
		return files;
		
	}
	
	public static DatasetFile getFileData(File file) {
		DatasetFile f = new DatasetFile();
		
		String filename = file.getName();
		long size = file.length();
		Format format = getFormat(file);
		ArrayList<Label> labels = getFileLabels(file, format,f);
		
		Set<Label> l = new HashSet<>();
		l.addAll(labels);
		
		f.setFileName(filename);
		f.setFormat(format);
		f.setSize(size);
		f.setLabels(l);
		
		
		return f;
	}
	
	public static Set<String> getDistributionContent(DatasetFile d,String user, String name){
		Set<String> set = new HashSet<>();
		File f = new File(UPLOADED_FOLDER + user + "/" + name+ "/" + d.getFileName() );
		
		switch(d.getFormat()) {
			case CSV:
				set.addAll(getCSVContent(f));
				break;
			case JSON:
				set.addAll(getJSONContent(f));
				break;
		}
		
		return set;
	}
	
	private static Set<String> getCSVContent(File f){
		Set<String> set = new HashSet<>();
		
		FileReader filereader;
		try {
			filereader = new FileReader(f);
			
			// create csvReader object passing 
	        // file reader as a parameter 
	        CSVReader csvReader = new CSVReader(filereader); 
	        String[] nextRecord = csvReader.readNext(); 
	        
	        while((nextRecord = csvReader.readNext()) != null) {
	        	for(String cell: nextRecord) {
	        		String aux = cleanCad(cell);
	        		set.add(aux);
	        	}
	        }
	        
	        System.out.println("Numero de datos distintos en el dataset "+f.getName()+"-- >" + set.size());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
     
		return set;
	}
	
	
	public static Set<String> getJSONContent(File f){
		Set<String> set = new HashSet<>();
		
		JSONParser parser = new JSONParser();
		
		try {

			Object obj = parser.parse( new FileReader(f));
			//Si empieza el fichero con [ es un JSONArray
			
			
			if (obj instanceof JSONArray) {
			    // It's an array
			    JSONArray jsonArray = (JSONArray) obj;
			    
			  //Lectura de datos
		    	
		    	Set<String> values = new HashSet<>();
		    	for(int i = 0; i < jsonArray.size(); i++) {
		    		values.addAll(getJSONValues((JSONObject)jsonArray.get(i)));
		    	}
		    	System.out.println("Numero de datos distintos en el dataset JSON Array-- >" + values.size());

			    
			}
			else if (obj instanceof JSONObject) {
			    // It's an object
				JSONObject jsonObject = (JSONObject) obj;
					
				//Lectura de datos!
				Set<String> values = new HashSet<>();
				values.addAll(getJSONValues(jsonObject));
				System.out.println("Numero de datos distintos en el dataset "+f.getName()+"-- >" + values.size());
				
				
			}

	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
		
		return set;
	}
	
	
	private static ArrayList<Label> getFileLabels(File file, Format format,DatasetFile f){
		ArrayList<Label> labels = new ArrayList<Label>();
		
		switch (format) {
			case CSV:
				labels = readLabelsCSV(file,f);
				break;
			case JSON:
				labels = readLabelsJSON(file,f);
				break;
		
		}
		
		
		return labels;
	}
	
	private static ArrayList<Label> readLabelsCSV(File file,DatasetFile f){
		ArrayList<Label> labels = new ArrayList<Label>();
		
		try { 
			  
	        // Create an object of filereader 
	        // class with CSV file as a parameter. 
	        FileReader filereader = new FileReader(file); 
	  
	        // create csvReader object passing 
	        // file reader as a parameter 
	        CSVReader csvReader = new CSVReader(filereader); 
	        String[] nextRecord = csvReader.readNext(); 
	        
	        if(nextRecord != null) {
	        	for(String cell: nextRecord) {
	        		Label l = new Label(cell, "","");
	        		l.setDatasetFile(f);
	        		labels.add(l);
	        	}
	        }
	    } 
	    catch (Exception e) { 
	        e.printStackTrace(); 
	    } 
		
		return labels;
	}
	
	private static ArrayList<Label> readLabelsJSON(File file,DatasetFile f){
		ArrayList<Label> labels = new ArrayList<Label>();
		JSONParser parser = new JSONParser();
		
		try {

			Object obj = parser.parse( new FileReader(file));
			//Si empieza el fichero con [ es un JSONArray
			
			
			if (obj instanceof JSONArray) {
			    // It's an array
			    JSONArray jsonArray = (JSONArray) obj;
			    
			    labels = readJSONArray(jsonArray,f);
			    System.out.println("Its an array");
			    
			}
			else if (obj instanceof JSONObject) {
			    // It's an object
				JSONObject jsonObject = (JSONObject) obj;
				
				labels = readJSONObject(jsonObject,f);
				
				//Lectura de datos!
				Set<String> values = new HashSet<>();
				values.addAll(getJSONValues(jsonObject));
				System.out.println("Numero de datos distintos en el dataset "+file.getName()+"-- >" + values.size());
				
				
			}

	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
		
		return labels;
	}
	
	private static ArrayList<String> getJSONValues(JSONObject obj) {
		Collection d = obj.values();
		ArrayList<String> cads = decodJSONValues(d);
		ArrayList<String> cleaned = new ArrayList<>();
		
		for(String c:  cads) {
			cleaned.add(cleanCad(c));
		}
 		
		return cleaned;
	}
	
	private static ArrayList<String> decodJSONValues(Collection c){
		ArrayList<String> decoded = new ArrayList<>();
		
		for(Object d: c) {
			if(d!=null) {
				String s = "";
				if(d instanceof Double || d instanceof Long) {
					s = String.valueOf(d);
				}
				else {
					s = (String) d;
				}
							
				decoded.add(s);
			}
		}
		
		
		return decoded;
	}
	
	private static ArrayList<Label> readJSONArray(JSONArray array,DatasetFile f){
		ArrayList<Label> labels = new ArrayList<Label>();
		
		if(array.size() > 0) {
	    	JSONObject json = (JSONObject) array.get(0);
	    	labels = readJSONObject(json,f);
	    }
	
		return labels;
	}
	
	private static ArrayList<Label> readJSONObject(JSONObject obj,DatasetFile f){
		ArrayList<Label> labels = new ArrayList<Label>();
		
		Set<Object> set = obj.keySet();

		Iterator<Object> iterator = set.iterator();

		while (iterator.hasNext()) {
			String key = (String) iterator.next();
			Label l = new Label(key, "","");
			l.setDatasetFile(f);
    		labels.add(l);
		}
		
		return labels;
	}
	
	
	private static Format getFormat(File file) {
		Format format = null;
		
		String mimeType = extractFileType(file);
		
		switch(mimeType) {
			//CSV
			case "text/csv":
				format = Format.CSV;
				break;
			//JSON
			case "application/json":
				format = Format.JSON;
				break;
//			//XML
//			case "application/xml":
//				format = Format.XML;
//				break;
//			//XLS	
//			case "application/vnd.ms-excel":
//				format = Format.XLS;
//				break;
		}
		
		return format;
	}
	
	public static String extractFileType(File file) {
		String type = "";
		
	    Tika tika = new Tika();
	    try {
			type = tika.detect(file);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return type;
	}
	
	private static String cleanCad(String cad) {
		String cleaned = "";
		
		cleaned = (cad.trim()).toLowerCase();
		
		return cleaned.trim();
	}
	
	public static ArrayList<ArrayList<String>> getSampleData (File f, int rows){
		ArrayList<ArrayList<String>> sampleData = new ArrayList<ArrayList<String>>();
		Format format = getFormat(f);
		
		
		switch (format) {
			case CSV:
				sampleData = getCSVContentRows(f, rows);
				break;
			case JSON:
						
				break;
	
			default:
				break;
		}
		
		return sampleData;
	}
	
	private static ArrayList<ArrayList<String>> getCSVContentRows(File f, int rows){
		ArrayList<ArrayList<String>> data = new ArrayList<ArrayList<String>>();
		
		FileReader filereader;
		try {
			filereader = new FileReader(f);
			
			// create csvReader object passing 
	        // file reader as a parameter 
	        CSVReader csvReader = new CSVReader(filereader); 
	        String[] nextRecord;
	        int n = 0;
	        
	        while((nextRecord = csvReader.readNext()) != null && n < rows) {
	        	ArrayList<String> row = new ArrayList<String>();
	        	for(String cell: nextRecord) {
	        		//String aux = cleanCad(cell);
	        		row.add(cell);
	        	}
	        	data.add(row);
	        	n++;
	        }

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
     
		return data;
	}
	
	public static String getJSONRows(File f, int rows) {
		String json = "";
		
		JSONParser parser = new JSONParser();
		
		try {

			Object obj = parser.parse( new FileReader(f));
			//Si empieza el fichero con [ es un JSONArray
			
			
			if (obj instanceof JSONArray) {
			    // It's an array
			    JSONArray jsonArray = (JSONArray) obj;
			    
			  //Lectura de datos
		    	
			    json+="[";
		    	for(int i = 0; i < jsonArray.size() && i < rows; i++) {
		    		
		    		json += readAllJSONObjectContent((JSONObject)jsonArray.get(i))+",";
		    	}
		    	json = json.substring(0, json.lastIndexOf(","));
		    	json+="]";
			    
			}
			else if (obj instanceof JSONObject) {
			    // It's an object
				JSONObject jsonObject = (JSONObject) obj;
					
				//Lectura de datos!

				 json = readAllJSONObjectContent(jsonObject);

				
				
			}

	    } catch (Exception ex) {
	        ex.printStackTrace();
	    }
		
		
		return json;
	}
	
	private static String readAllJSONObjectContent(JSONObject ob) {
		
		String s =  ob.toJSONString();
		
		
		
		return s;
	}
}
